__author__ = 'Deza Farras Tsany<deza.ftsany@gmail.com>'
__version__ = '1.0.0'

"""
This module offers random password generation.
"""

from random import choice, randint, shuffle
from string import digits, punctuation, ascii_letters
from .safe import check

_min_length = 5
_max_length = 8
_min_digit_length = 7
_max_digit_length = 12
_min_word_length = 9
_max_word_length = 15

def _randomDigits():
    length = randint(_min_length, _max_length)
    return ''.join(choice(digits) for i in range(length))

def _randomPunctuation():
    length = randint(_min_digit_length, _max_digit_length)
    return ''.join(choice(punctuation) for i in range(length))

def _randomWord():
    length = randint(_min_word_length, _max_word_length)
    return ''.join(choice(ascii_letters) for i in range(length))



class generate():

    def __init__(self) :
        pass

    def __response(self, password='', level=0, strength='', message='') :
        return {
            'password': password,
            'level': level,
            'strength': strength,
            'message': message
        }

    def number(self, min=6, max=6) :
        length = randint(min, max)
        str_pass = ''.join(choice(digits) for i in range(length))
        checkPassword = check(str_pass)
        return self.__response(str_pass, level=checkPassword['level'], strength=checkPassword['strength'], message=checkPassword['message']) 

    def __punctuation(self, min=6, max=6) :
        length = randint(min, max)
        str_pass = ''.join(choice(punctuation) for i in range(length))
        checkPassword = check(str_pass)
        return self.__response(str_pass, level=checkPassword['level'], strength=checkPassword['strength'], message=checkPassword['message']) 
    
    def letter(self, min=6, max=6) :
        length = randint(min, max)
        str_pass = ''.join(choice(ascii_letters) for i in range(length))
        checkPassword = check(str_pass)
        return self.__response(str_pass, level=checkPassword['level'], strength=checkPassword['strength'], message=checkPassword['message'])
    
    def numberLetter(self):
        pass_number = self.number(min=3, max=5)
        pass_letter = self.letter(min=3, max=5)
        
        str_pass = pass_number['password'] +  pass_letter['password']
        str_pass = list(str_pass)
        shuffle(str_pass)
        str_pass = ''.join( str_pass )

        checkPassword = check(str_pass)
        
        return self.__response(str_pass, level=checkPassword['level'], strength=checkPassword['strength'], message=checkPassword['message'])

    def random(self) :
        """
            This function aims to generate passwords randomly
        """
        pass_number = self.number(_min_digit_length, _max_digit_length)
        pass_punctuation = self.__punctuation(_min_length, _max_length)
        pass_letter = self.letter(_min_word_length, _max_word_length)
        
        str_pass = pass_number['password'] + pass_punctuation['password'] + pass_letter['password']
        str_pass = list(str_pass)
        shuffle(str_pass)
        str_pass = ''.join( str_pass )

        checkPassword = check(str_pass)
        
        return self.__response(str_pass, level=checkPassword['level'], strength=checkPassword['strength'], message=checkPassword['message'])


# coding: utf-8
"""
    safe._compat

    Compatible module for Python 2 and Python 3.

    :copyright: (c) 2014 by Hsiaoming Yang
"""


import sys
try:
    import cPickle as pickle
except ImportError:
    import pickle

if sys.version_info[0] == 3:
    unicode_type = str
    bytes_type = bytes
else:
    unicode_type = unicode
    bytes_type = str


__all__ = ['pickle', 'to_unicode']


def to_unicode(value, encoding='utf-8'):
    if isinstance(value, unicode_type):
        return value

    if isinstance(value, bytes_type):
        return unicode_type(value, encoding=encoding)

    if isinstance(value, int):
        return unicode_type(str(value))

    return value



# coding: utf-8
import os
import tempfile

import safe

def test_asdf():
    s = safe.check('dfghjkl', length=7)
    assert not s
    assert 'pattern' in str(s)


def test_step():
    s = safe.check('abcdefg', length=7)
    assert not s
    assert 'pattern' in s.message


def test_common():
    s = safe.check('password')
    assert not s
    assert 'common' in s.message


def test_short():
    s = safe.check('1')
    assert not s
    assert 'short' in s.message


def test_simple():
    s = safe.check('yhnolku', length=6)
    assert not s
    assert 'simple' in s.message


def test_medium():
    s = safe.check('yhnolkuT', length=7)
    assert not s
    assert repr(s) == 'medium'


def test_strong():
    s = safe.check('yhnolkuT.')
    assert bool(s)
    assert 'perfect' in s.message


def test_no_cache_on_load_words():
    cache_file = _clear_cache_file()
    words = safe._load_words(cache_words=False)
    assert words
    assert not os.path.exists(cache_file)


def test_no_cache_on_safe_check():
    cache_file = _clear_cache_file()
    s = safe.check('yhnolkuT.', cache_words=False)
    assert not os.path.exists(cache_file)

def _clear_cache_file():
    filename = 'safe-%s.words.cache' % safe.__version__
    _cache_file = os.environ.get(
        'PYTHON_SAFE_WORDS_CACHE',
        os.path.join(tempfile.gettempdir(), filename),
    )
    if os.path.exists(_cache_file):
        os.remove(_cache_file)
    return _cache_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    import multiprocessing
except ImportError:
    pass

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from email.utils import parseaddr
import safe

author, author_email = parseaddr(safe.__author__)


def fread(filename):
    with open(filename) as f:
        return f.read()


setup(
    name='Safe',
    version=safe.__version__,
    author=author,
    author_email=author_email,
    url='https://github.com/lepture/safe',
    packages=["safe"],
    description="Is your password safe?",
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    long_description=fread('README.rst'),
    license='BSD',
    install_requires=[],
    tests_require=['nose'],
    test_suite='nose.collector',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)


from setuptools import setup, find_packages, Extension
# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
  name = 'rand_pass_generator',         # How you named your package folder (MyLib)
  packages = ['rand_pass_generator', 'rand_pass_generator.safe', 'rand_pass_generator.safe.safe'],   # Chose the same as "name"
  version = '1.0.26',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'This module offers random password generator',   # Give a short description about your library
  long_description=long_description,            # Give a long description about your library
  long_description_content_type='text/markdown',
  author = 'Deza Farras Tsany',                   # Type in your name
  author_email = 'deza.ftsany@gmail.com',      # Type in your E-Mail
  url = 'https://defartsa.dev/',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/defartsa23/password-generator-py/archive/refs/heads/main.zip',    # I explain this later on
  project_urls={
    'Documentation': 'https://github.com/defartsa23/password-generator-py',
    'Source': 'https://github.com/defartsa23/password-generator-py',
    'Tracker': 'https://github.com/defartsa23/password-generator-py/issues',
  }, 
  keywords = ['PASSWORD', 'GENERATOR', 'RANDOM'],   # Keywords that define your package best
  install_requires=[ ],
  dependency_links=[ ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3 :: Only',
    'Operating System :: MacOS',
    'Operating System :: Microsoft',
    'Operating System :: Unix'
  ],
  include_package_data=True,
  zip_safe=False,
  setup_requires=['pytest-runner'],
  tests_require=['pytest'],
  test_suite='test'
)


# coding: utf-8
"""
    safe
    ~~~~

    Detect if your password is safe.

    :copyright: (c) 2014 by Hsiaoming Yang
"""

import re
import logging
import os.path
import tempfile
from ._compat import to_unicode, pickle

__version__ = '0.5'
__author__ = 'Hsiaoming Yang <me@lepture.com>'

__all__ = [
    'is_asdf', 'is_by_step', 'is_common_password',
    'check', 'Strength',
]

log = logging.getLogger('safe')

LOWER = re.compile(r'[a-z]')
UPPER = re.compile(r'[A-Z]')
NUMBER = re.compile(r'[0-9]')
MARKS = re.compile(r'[^0-9a-zA-Z]')
SYMBOL = re.compile(r"[ !#$%&'()*+,-./[\\\]^_`{|}~"+r'"]')
NUM_ASC = '01234567890'
NUM_DESC = '09876543210'

TERRIBLE = 0
SIMPLE = 1
MEDIUM = 2
STRONG = 3
VERYSTRONG = 4 


def _load_words(cache_words=True):
    if cache_words:
        filename = 'safe-%s.words.cache' % __version__
        _cache_file = os.environ.get(
            'PYTHON_SAFE_WORDS_CACHE',
            os.path.join(tempfile.gettempdir(), filename),
        )

        if os.path.exists(_cache_file):
            log.debug('Reading from cache file %s' % _cache_file)
            try:
                with open(_cache_file, 'rb') as f:
                    return pickle.load(f)
            except:
                pass

    filepath = os.environ.get(
        'PYTHON_SAFE_WORDS_FILE',
        os.path.join(os.path.dirname(__file__), 'words.dat'),
    )
    words = {}
    with open(filepath, 'rb') as f:
        for line in f.readlines():
            name, freq = line.split()
            words[to_unicode(name.strip())] = int(freq.strip())
    if cache_words:
        with open(_cache_file, 'wb') as f:
            log.debug('Dump to cache file %s' % _cache_file)
            pickle.dump(words, f)
    return words

WORDS = {}
ASDF = ['qwertyuiop', 'asdfghjkl', 'zxcvbnm']


def is_asdf(raw):
    """If the password is in the order on keyboard."""

    reverse = raw[::-1]
    asdf = ''.join(ASDF)

    return raw in asdf or reverse in asdf


def is_by_step(raw):
    """If the password is alphabet step by step."""
    # make sure it is unicode
    delta = ord(raw[1]) - ord(raw[0])

    for i in range(2, len(raw)):
        if ord(raw[i]) - ord(raw[i-1]) != delta:
            return False

    return True


def is_common_password(raw, freq=0, cache_words=True):
    """If the password is common used.

    10k top passwords: https://xato.net/passwords/more-top-worst-passwords/
    """
    global WORDS
    if not WORDS:
        WORDS = _load_words(cache_words)
    frequent = WORDS.get(raw, 0)
    if freq:
        return frequent > freq
    return bool(frequent)


class Strength(object):
    """Measure the strength of a password.

    Here are some common usages of strength::

        >>> strength = Strength(True, 3, 'strong', 'password is perfect')
        >>> bool(strength)
        True
        >>> repr(strength)
        'strong'
        >>> str(strength)
        'password is perfect'

    :param valid: if the password is valid to use
    :param level: the level of the password
    :param strength: the strength level of the password
    :param message: a message related to the password
    """
    
    def __init__(self, valid, level, strength, message):
        self.valid = valid
        self.level = level
        self.strength = strength
        self.message = message

    def __repr__(self):
        return self.strength

    def __str__(self):
        return self.message

    def __unicode__(self):
        return self.message

    def __nonzero__(self):
        return self.valid

    def __bool__(self):
        return self.valid


def check(raw, length=8, freq=0, min_types=5, min_symbol=3, min_number=3, level=STRONG, cache_words=True):
    """Check the safety level of the password.

    :param raw: raw text password.
    :param length: minimal length of the password.
    :param freq: minimum frequency.
    :param min_types: minimum character family.
    :param level: minimum level to validate a password.
    """
    raw = to_unicode(raw)
    
    if level > STRONG:
        level = STRONG

    if len(raw) < length:
        return Strength(False, TERRIBLE, 'terrible', 'password is too short').__dict__

    if is_asdf(raw) or is_by_step(raw):
        return Strength(False, SIMPLE, 'simple', 'password has a pattern').__dict__

    if is_common_password(raw, freq=freq, cache_words=cache_words):
        return Strength(False, SIMPLE, 'simple', 'password is too common').__dict__

    types = 0

    if LOWER.search(raw):
        types += 1

    if UPPER.search(raw):
        types += 1

    if NUMBER.search(raw):
        types += 1

    if MARKS.search(raw):
        types += 1
    
    if len(MARKS.findall(raw)) >= min_symbol:
        types += 2
    
    if len(NUMBER.findall(raw)) >= min_number:
        types += 2
        
    # Find all the numbe ascending
    index = 2
    while index < len(raw):
        temp_string = raw[index-2] + raw[index-1] + raw[index]
        if NUM_ASC.find(temp_string) != -1:
            types -= 1
            break
        
        index += 1

    # Find all the numbe descending
    index = 2
    while index < len(raw):
        temp_string = raw[index-2] + raw[index-1] + raw[index]
        if NUM_DESC.find(temp_string) != -1:
            types -= 1
            break
        
        index += 1  
    
    if types < 2:
        return Strength(level <= SIMPLE, SIMPLE, 'simple', 'password is too simple').__dict__

    if types < min_types:
        return Strength(level <= MEDIUM, MEDIUM, 'medium',
                        'password is good enough, but not strong').__dict__

    if types > 7:
        return Strength(True, VERYSTRONG, 'very strong', 'password is very perfect').__dict__
    return Strength(True, STRONG, 'strong', 'password is perfect').__dict__


def safety(raw, length=8, freq=0, min_types=2, level=STRONG):
    return check(raw, length=8, freq=0, min_types=2, level=STRONG)
